@@ -13,7 +13,7 @@ if(length(num.df)==1) num.df<-rep(num.df,ncol(LRT))
 ### shrunken point estimates of dispersion 
 
 #### Code for implementing Smyth's approach begins here ####
-	shrink.phi<-function(phi.hat,den.df){
+shrink.phi<-function(phi.hat,den.df){
 	phi.hat[phi.hat<=0]<-min(phi.hat[phi.hat>0])
 	z<-log(phi.hat); z[z==Inf]<-max(z[z!=Inf]); z[z==-Inf]<-min(z[z!=-Inf]);mnz<-mean(z)
 
@@ -26,9 +26,11 @@ if(length(num.df)==1) num.df<-rep(num.df,ncol(LRT))
 		phi0<-exp(mnz-digamma((den.df)/2)+digamma(d0/2)- log(d0/(den.df)))
 
 		## compute shrunken phi's
-		phi.shrink<-((den.df)*phi.hat+d0*phi0)/(den.df+d0)  }
+		phi.shrink<-((den.df)*phi.hat+d0*phi0)/(den.df+d0)  
+	}
 	else{phi.shrink<-rep(exp(mnz),length(z)); d0<-Inf; phi0<-exp(mnz) }
-	return(list(phi.shrink=phi.shrink,d0=d0,phi0=phi0))  }
+	return(list(phi.shrink=phi.shrink,d0=d0,phi0=phi0))  
+}
 #### Code for implementing Smyth's approach ends here ####
 
 phi.hat[phi.hat<0]<-min(phi.hat[phi.hat>0])
@@ -41,68 +43,81 @@ if(Model=="Poisson") phi.shrink[phi.shrink<1]<-1
 
 ### Fit cubic spline to capture relationship between log(y.bar) and log(phi.hat)
 y<-log(phi.hat); y[y==-Inf]<-min(y[y!=-Inf]); y[y==Inf]<-max(y[y!=Inf])
-if(is.null(spline.df)) spline.fit<-sreg(log(mn.cnt), y)
-else spline.fit<-sreg(log(mn.cnt), y,df=spline.df)
+
+spline.fit<-if(is.null(spline.df)) smooth.spline(x=log(mn.cnt), y=y) else smooth.spline(x=log(mn.cnt), y=y, df= spline.df)
+spline.pred<-predict(spline.fit,x=log(mn.cnt))$y
+
+fit.method<-"spline"
+
 
 ### Obtain estimate for prior degrees of freedom and scaling factor after adjusting for cubic spline fit
-y2<-phi.hat/exp(spline.fit$fitted.values)
+y2<-phi.hat/exp(spline.pred)
 shrink<-shrink.phi(y2,den.df)
 D0<-shrink[[2]]; 
-phi0<-shrink[[3]]; print(paste("Spline scaling factor:",phi0))
+phi0<-shrink[[3]]; print(paste("Spline scaling factor:",phi0)) 
 
 
 ### If desired, plot resulting cubic spline fit
 if(Plot){
-	x11(height=9); nf <- layout(matrix(1:2, 2, 1),heights=c(7,2))
+	dev.new(height=9); nf <- layout(matrix(1:2, 2, 1),heights=c(7,2))
 
-par(mai=c(1,1.2,1,.2))
+	par(mai=c(1,1.2,1,.2))
 	suppressWarnings(plot(log(mn.cnt),y,xlab=expression(log(bar(y)[phantom()%.%phantom()]*phantom()[phantom()%.%phantom()])),
-ylab=expression(log(hat(Phi))),main="Estimated Dispersion
+	ylab=expression(log(hat(Phi))),main="Estimated Dispersion
  versus Average Count",pch=1,cex.lab=2,cex.axis=2,cex.main=2))
-	lines(sort(log(mn.cnt)),spline.fit$fitted.values[order(mn.cnt)],col=2,lwd=3)
+	lines(sort(log(mn.cnt)),spline.pred[order(mn.cnt)],col=2,lwd=3)
 
-###Compare quantiles from estimated theoretical and empirical dispersion distributions
+	###Compare quantiles from estimated theoretical and empirical dispersion distributions
 
 
-RR<-NULL; sort.mn<-log(sort(mn.cnt)); qq<-c(.05,.95);ord.F<-y[order(mn.cnt)]
-bins<-c(1+round(length(y)*(0:19)/20),length(y))
-for(ii in 1:(length(bins)-1)){
-ind<-bins[ii]:bins[ii+1]
-RR<-rbind(RR,c(quantile(ord.F[ind],qq),median(sort.mn[ind])))
-}
-
-lines(sort(log(mn.cnt)),spline.fit$fitted.values[order(mn.cnt)]+log(phi0*qf(.95,den.df,D0)),col=4,lwd=3)
-lines(sort(log(mn.cnt)),spline.fit$fitted.values[order(mn.cnt)]+log(phi0*qf(.05,den.df,D0)),col=4,lwd=3)
-
-lines(RR[,3],RR[,2],col=3,lwd=3)
-lines(RR[,3],RR[,1],col=3,lwd=3)
-par(mai=rep(0,4));plot(19,19,col="white",axes=FALSE)
-
-suppressWarnings(legend("top",legend=c(paste("Fitted Spline with",signif(spline.fit$eff.df,2),"df"),
-"0.05 & 0.95 Quantiles from Empirical Distribution",
-"0.05 & 0.95 Quantiles from Scaled F-Distribution"),lwd=3,lty=1,col=2:4,cex=1.5))
+	RR<-NULL; sort.mn<-log(sort(mn.cnt)); qq<-c(.05,.95);ord.F<-y[order(mn.cnt)]
+	bins<-c(1+round(length(y)*(0:19)/20),length(y))
+	for(ii in 1:(length(bins)-1)){
+		ind<-bins[ii]:bins[ii+1]
+		RR<-rbind(RR,c(quantile(ord.F[ind],qq),median(sort.mn[ind])))
+	}
+	
+	lines(sort(log(mn.cnt)),spline.pred[order(mn.cnt)]+log(phi0*qf(.95,den.df,D0)),col=4,lwd=3)
+	lines(sort(log(mn.cnt)),spline.pred[order(mn.cnt)]+log(phi0*qf(.05,den.df,D0)),col=4,lwd=3)
+	
+	lines(RR[,3],RR[,2],col=3,lwd=3)
+	lines(RR[,3],RR[,1],col=3,lwd=3)
+	par(mai=rep(0,4));plot(19,19,col="white",axes=FALSE)
+	
+	suppressWarnings(legend("top",legend=c(paste("Fitted", fit.method,"with",signif(spline.fit$df,2),"df"),
+	"0.05 & 0.95 Quantiles from Empirical Distribution",
+	"0.05 & 0.95 Quantiles from Scaled F-Distribution"),lwd=3,lty=1,col=2:4,cex=1.5))
 }
 
 
 
-phi.spline<-(D0*exp(spline.fit$fitted.values)*phi0+(den.df)*phi.hat)/(D0+den.df)
-if(D0==Inf){ warning("D0 estimate is infinity for QLSpline (there's little scatter in original dispersion estimates around fitted spline).
+phi.spline<-(D0*exp(spline.pred)*phi0+(den.df)*phi.hat)/(D0+den.df)
+if(D0==Inf){ 
+	warning("D0 estimate is infinity for QLSpline (there's little scatter in original dispersion estimates around fitted spline).
  QLSpline dispersions set to fitted cubic spline (use 'Plot=TRUE' to view) with no uncertainty.")
-phi.spline<-exp(spline.fit$fitted.values)
+	phi.spline<-exp(spline.pred)
 }
 if(Model=="Poisson") phi.spline[phi.spline<1]<-1
 
-pval<-list(QL=NULL,QLShrink=NULL,QLSpline=NULL)
+log.pval<-F.stat<-list(QL=NULL,QLShrink=NULL,QLSpline=NULL)
 
 for(i in 1:ncol(LRT)){
-#### Compute p-values for each hypothesis test using each approach
-pval[[1]]<-cbind(pval[[1]],1-pf(LRT[,i]/phi.hat2,num.df[i],den.df))
-pval[[2]]<-cbind(pval[[2]],1-pf(LRT[,i]/phi.shrink,num.df[i],est.d0+den.df))
-pval[[3]]<-cbind(pval[[3]],1-pf(LRT[,i]/phi.spline,num.df[i],D0+den.df))
+	#### Compute p-values for each hypothesis test using each approach
+	log.pval[[1]]<-cbind(log.pval[[1]],pf(LRT[,i]/phi.hat2,num.df[i],den.df,lower.tail = FALSE,log.p=TRUE))
+	log.pval[[2]]<-cbind(log.pval[[2]],pf(LRT[,i]/phi.shrink,num.df[i],est.d0+den.df,lower.tail = FALSE,log.p=TRUE))
+	log.pval[[3]]<-cbind( log.pval[[3]], pf(LRT[,i]/phi.spline, num.df[i], D0+den.df, lower.tail = FALSE, log.p=TRUE))
+
+	F.stat[[1]]<-cbind(F.stat[[1]],LRT[,i]/phi.hat2)
+	F.stat[[2]]<-cbind(F.stat[[2]],LRT[,i]/phi.shrink)
+	F.stat[[3]]<-cbind(F.stat[[3]],LRT[,i]/phi.spline)
 }
 
-for(ii in 1:3) colnames(pval[[ii]])<-colnames(LRT)
-d0<-c(est.d0,D0);names(d0)<-c("QLShrink","QLSpline")
+pval<-log.pval
+for(ii in 1:3){
+	pval[[ii]]<-exp(log.pval[[ii]])
+	colnames(F.stat[[ii]])<-colnames(log.pval[[ii]])<-colnames(pval[[ii]])<-colnames(LRT)
+}
+d0<-c(QLShrink=est.d0, QLSpline=D0)
 
 estimate.m0<-function(p, B = 20){
 
@@ -161,9 +176,9 @@ for(ii in 1:length(qval)){
 	m0<-rbind(m0,M0)
 }
 
-colnames(m0)<-colnames(qval[[1]]); rownames(m0)<-names(pval)
+colnames(m0)<-colnames(qval[[1]]); rownames(m0)<-names(log.pval)
 
-return(list(P.values=pval,Q.values=qval,m0=m0,d0=d0))
+return(list(P.values=pval,log.P.values=log.pval,Q.values=qval,F.stat=F.stat,m0=m0,d0=d0))
 }
 
 
